@@ -24,12 +24,9 @@ plainbox.impl.test_session
 Test definitions for plainbox.impl.session module
 """
 
-import json
-
 from unittest import TestCase, expectedFailure
 
 from plainbox.abc import IJobResult
-from plainbox.impl.result import MemoryJobResult
 from plainbox.impl.session import JobReadinessInhibitor
 from plainbox.impl.session import JobState
 from plainbox.impl.session import UndesiredJobReadinessInhibitor
@@ -145,9 +142,9 @@ class JobStateTests(TestCase):
 
     @expectedFailure
     def test_setting_job_is_not_allowed(self):
-        #FIXME: We want this test to come back at some point so I didn't
-        #delete it, but at the moment we need it to always pass because
-        #a JobState's job attribute needs to be writable.
+        # FIXME: We want this test to come back at some point so I didn't
+        # delete it, but at the moment we need it to always pass because
+        # a JobState's job attribute needs to be writable.
         with self.assertRaises(AttributeError):
             self.job_state.job = None
 
